import dayjs from "dayjs";
import editForm from "../components/form.vue";
import { message } from "@/utils/message";
import { type Ref, ref, reactive, onMounted, h, toRaw } from "vue";
import type { HtsClassInfo, HtsItemInfo, VersionInfo } from "types/hts";
import type { PaginationProps } from "@pureadmin/table";
import { addDialog } from "@/components/ReDialog";
import {
  deleteHtsItemAPI,
  deleteHtsItemListAPI,
  getHtsClassListAPI,
  getHtsItemListAPI,
  getVersionListAPI,
  postAddHtsItemAPI,
  postImportHtsItemAPI,
  putUpdateHtsItemAPI
} from "@/api/hts";
import { getKeyList } from "@pureadmin/utils";
import { ElMessageBox, type UploadUserFile } from "element-plus";
import { deleteFileAPI, postUploadFileAPI } from "@/api/file";
import { $t, transformI18n } from "@/plugins/i18n";

export const useHtsItem = (tableRef: Ref) => {
  /**
   * 查询表单
   */
  const form = reactive({
    /** 编码 */
    htsno: "",

    /** 描述 */
    description: "",

    /** 单位列表 */
    units: "",

    /** 通用税率 */
    general: "",

    /** 特殊税率，适用于特定国家或地区 */
    special: "",

    /** 其他税率 */
    other: "",

    /** 配额数量 */
    quota_quantity: "",

    /** 附加税 */
    additional_duties: "",

    /** 脚注列表 */
    footnotes: "",

    /** 所属类 */
    class_id: "",

    /** 所属版本 */
    version_id: ""
  });
  /**
   * 表单Ref
   */
  const formRef = ref(null);
  /**
   * 数据列表
   */
  const dataList = ref<HtsItemInfo[]>([]);
  /**
   * 加载状态
   */
  const loading = ref(true);
  /**
   * 已选数量
   */
  const selectedNum = ref<number>(0);
  /**
   * 分页参数
   */
  const pagination = reactive<PaginationProps>({
    total: 0,
    pageSize: 10,
    currentPage: 1,
    background: true,
    pageSizes: [10, 20, 30, 40, 50]
  });

  /**编码类别列表 */
  const classList = ref<HtsClassInfo[]>([]);

  /**编码版本列表 */
  const versionList = ref<VersionInfo[]>([]);
  // 上传文件区域显示状态
  const showUploadArea = ref(false);
  const fileList = ref<UploadUserFile[]>([]);
  /**上传成功后文件列表 */
  const fileIds = ref([]);
  const fileId = ref<string>("");
  /**上传按钮状态 */
  const uploadStatus = ref<boolean>(false);
  const version_id = ref<string>("");
  const class_id = ref<string>("");
  /**
   * 表格列设置
   */
  const columns: TableColumnList = [
    {
      label: transformI18n($t("tag.checkColumn")), // 如果需要表格多选，此处label必须设置
      type: "selection",
      fixed: "left",
      reserveSelection: true // 数据刷新后保留选项
    },

    {
      label: transformI18n($t("HtsItem.tag.parent_id")),
      prop: "parent_id",
      hide: true
    },

    {
      label: transformI18n($t("HtsItem.tag.htsno")),
      prop: "htsno",
      hide: false,
      formatter: ({ htsno }) => {
        return htsno ? htsno.replace(/(\d{2})/g, "$1.").slice(0, -1) : "";
      }
    },

    {
      label: transformI18n($t("HtsItem.tag.indent")),
      prop: "indent",
      hide: true
    },

    {
      label: transformI18n($t("HtsItem.tag.description")),
      prop: "description",
      hide: false
    },

    {
      label: transformI18n($t("HtsItem.tag.units")),
      prop: "units",
      hide: false
    },

    {
      label: transformI18n($t("HtsItem.tag.general")),
      prop: "general",
      hide: false
    },

    {
      label: transformI18n($t("HtsItem.tag.special")),
      prop: "special",
      hide: false
    },

    {
      label: transformI18n($t("HtsItem.tag.other")),
      prop: "other",
      hide: false
    },

    {
      label: transformI18n($t("HtsItem.tag.quota_quantity")),
      prop: "quota_quantity",
      hide: false
    },

    {
      label: transformI18n($t("HtsItem.tag.additional_duties")),
      prop: "additional_duties",
      hide: false
    },

    {
      label: transformI18n($t("HtsItem.tag.footnotes")),
      prop: "footnotes",
      hide: true
    },

    {
      label: transformI18n($t("HtsItem.tag.class_")),
      prop: "class_id",
      hide: true
    },

    {
      label: transformI18n($t("HtsItem.tag.version")),
      prop: "version_id",
      hide: true
    },

    {
      label: transformI18n($t("tag.createTime")),
      prop: "create_time",
      hide: true,
      formatter: ({ create_time }) =>
        dayjs(create_time).format("YYYY-MM-DD HH:mm:ss")
    },

    {
      label: transformI18n($t("tag.updateTime")),
      prop: "update_time",
      hide: false,
      formatter: ({ update_time }) =>
        dayjs(update_time).format("YYYY-MM-DD HH:mm:ss")
    },

    {
      label: transformI18n($t("buttons.Operation")),
      fixed: "right",
      width: 220,
      slot: "operation"
    }
  ];
  /**
   * 初次查询
   */
  const onSearch = async () => {
    loading.value = true;
    const res = await getHtsItemListAPI({
      page: pagination.currentPage,
      pageSize: pagination.pageSize,
      ...toRaw(form),
      htsno: form.htsno.replace(/\D/g, "") || ""
    });
    if (res.success) {
      dataList.value = res.data.result;
      pagination.total = res.data.total;
      pagination.currentPage = res.data.page;
      pagination.pageSize = res.data.pageSize;
    }
    message(res.msg, {
      type: res.success ? "success" : "error"
    });
    loading.value = false;
  };
  /**
   * 重置表单
   * @param formEl 表单ref
   * @returns
   */
  const resetForm = async (formEl: any) => {
    if (!formEl) return;
    formEl.resetFields();
    await onSearch();
  };
  /**
   * 处理删除
   * @param row
   */
  const handleDelete = async (row: HtsItemInfo) => {
    const res = await deleteHtsItemAPI(row.id);
    if (res.success) {
      onSearch();
    }
    message(res.msg, {
      type: res.success ? "success" : "error"
    });
  };
  /**
   * 处理每页数量变化
   */
  const handleSizeChange = async (val: number) => {
    loading.value = true;
    const res = await getHtsItemListAPI({
      page: pagination.currentPage,
      pageSize: val,
      ...toRaw(form),
      htsno: form.htsno.replace(/\D/g, "") || ""
    });
    if (res.success) {
      dataList.value = res.data.result;
      pagination.total = res.data.total;
      pagination.currentPage = res.data.page;
      pagination.pageSize = res.data.pageSize;
    }
    message(res.msg, {
      type: res.success ? "success" : "error"
    });
    loading.value = false;
  };

  /**
   * 处理页码变化
   * @param val
   */
  const handleCurrentChange = async (val: number) => {
    loading.value = true;
    const res = await getHtsItemListAPI({
      page: val,
      pageSize: pagination.pageSize,
      ...toRaw(form),
      htsno: form.htsno.replace(/\D/g, "") || ""
    });
    if (res.success) {
      dataList.value = res.data.result;
      pagination.total = res.data.total;
      pagination.currentPage = res.data.page;
      pagination.pageSize = res.data.pageSize;
    }
    message(res.msg, {
      type: res.success ? "success" : "error"
    });
    loading.value = false;
  };
  /** 当CheckBox选择项发生变化时会触发该事件 */
  const handleSelectionChange = async (val: any) => {
    selectedNum.value = val.length;
    // 重置表格高度
    tableRef.value.setAdaptive();
  };

  /** 取消选择 */
  const onSelectionCancel = async () => {
    selectedNum.value = 0;
    // 用于多选表格，清空用户的选择
    tableRef.value.getTableRef().clearSelection();
  };
  /**
   * 批量删除
   */
  const onbatchDel = async () => {
    // 返回当前选中的行
    const curSelected = tableRef.value.getTableRef().getSelectionRows();
    const res = await deleteHtsItemListAPI(getKeyList(curSelected, "id"));
    if (res.success) {
      message(res.msg, {
        type: "success"
      });
      tableRef.value.getTableRef().clearSelection();
      onSearch();
    } else {
      message(res.msg, { type: "error", duration: 5000 });
    }
  };
  /** 上传文件前 */
  const beforeUpload = async file => {
    const isJSON =
      file.type === "application/json" || // 标准 JSON MIME 类型
      file.name.endsWith(".json"); // 兼容部分浏览器

    if (!isJSON) {
      message("只能上传 JSON 文件!", { type: "error" });
      return false;
    }

    return true;
  };

  /**处理文件上传 */
  const handleUpload = async () => {
    if (fileList.value.length === 0) {
      message("请先上传文件！", { type: "error", duration: 5000 });
      return;
    }
    uploadStatus.value = true;
    loading.value = true;
    for (const file of fileList.value) {
      if (file.status === "success") {
        if (!version_id.value) {
          message("请选择版本！", { type: "error" });
          return;
        }
        if (!class_id.value) {
          message("请选择分类！", { type: "error" });
          return;
        }

        const data = await postImportHtsItemAPI({
          file_id: fileId.value,
          version_id: version_id.value,
          class_id: class_id.value
        });
        if (data.success) {
          message("添加成功！", { type: "success" });
          onSearch();
        }
        continue;
      }
      try {
        const res = await postUploadFileAPI({
          file: file.raw
        });
        if (res.success) {
          file.status = "success";
          fileId.value = res.data.id;
          message(`${res.data.name}上传成功！`, { type: "success" });
          fileIds.value.push(res.data);
          if (!version_id.value) {
            message("请选择版本！", { type: "error" });
            return;
          }
          if (!class_id.value) {
            message("请选择分类！", { type: "error" });
            return;
          }
          const data = await postImportHtsItemAPI({
            file_id: fileId.value,
            version_id: version_id.value,
            class_id: class_id.value
          });
          if (data.success) {
            message(`添加成功！`, { type: "success" });
            onSearch();
          }
        } else {
          file.status = "fail";
        }
      } catch (error) {
        console.error(error);
      }
    }
    uploadStatus.value = false;
    loading.value = false;
  };
  /** 移除文件 */
  const beforeRemove = async uploadFile => {
    return ElMessageBox.confirm(`是否移除 ${uploadFile.name} ?`).then(
      async () => {
        if (uploadFile.status === "success") {
          const fileObj = fileIds.value.find(
            item => item.name === uploadFile.name
          );
          if (fileObj) {
            const res = await deleteFileAPI(fileObj.id);
            if (res.success) {
              message(res.msg, { type: "success" });
              fileIds.value = fileIds.value.filter(
                item => item.id !== fileObj.id
              );
              return true;
            } else {
              message(res.msg, { type: "error" });
              return false;
            }
          }
        }
        return true;
      }
    );
  };
  const openDialog = async (title = "Add", row?: HtsItemInfo) => {
    addDialog({
      title: transformI18n($t(`buttons.${title}`)),
      props: {
        formInline: {
          /** 方式 */
          title: title,

          /** 主键 */
          id: row?.id ?? "",

          /** 创建时间 */
          create_time: row?.create_time ?? "",

          /** 更新时间 */
          update_time: row?.update_time ?? "",

          /** 父编码 */
          parent_id: row?.parent_id ?? "",

          /** 编码 */
          htsno: row?.htsno ?? "",

          /** 层级 */
          indent: row?.indent ?? "",

          /** 描述 */
          description: row?.description ?? "",

          /** 单位列表 */
          units: row?.units ?? "",

          /** 通用税率 */
          general: row?.general ?? "",

          /** 特殊税率，适用于特定国家或地区 */
          special: row?.special ?? "",

          /** 其他税率 */
          other: row?.other ?? "",

          /** 配额数量 */
          quota_quantity: row?.quota_quantity ?? "",

          /** 附加税 */
          additional_duties: row?.additional_duties ?? "",

          /** 脚注列表 */
          footnotes: row?.footnotes ?? "",

          /** 所属类 */
          class_id: row?.class_id ?? "",

          /** 所属版本 */
          version_id: row?.version_id ?? "",
          versionList: versionList.value ?? [],
          classList: classList.value ?? []
        }
      },
      width: "45%",
      draggable: true,
      fullscreenIcon: true,
      closeOnClickModal: false,
      contentRenderer: () =>
        h(editForm, {
          formInline: {
            // /** 方式 */
            // title: title,

            /** 主键 */
            id: row?.id ?? "",

            /** 创建时间 */
            create_time: row?.create_time ?? "",

            /** 更新时间 */
            update_time: row?.update_time ?? "",

            /** 父编码 */
            parent_id: row?.parent_id ?? "",

            /** 编码 */
            htsno: row?.htsno ?? "",

            /** 层级 */
            indent: row?.indent ?? "",

            /** 描述 */
            description: row?.description ?? "",

            /** 单位列表 */
            units: row?.units ?? "",

            /** 通用税率 */
            general: row?.general ?? "",

            /** 特殊税率，适用于特定国家或地区 */
            special: row?.special ?? "",

            /** 其他税率 */
            other: row?.other ?? "",

            /** 配额数量 */
            quota_quantity: row?.quota_quantity ?? "",

            /** 附加税 */
            additional_duties: row?.additional_duties ?? "",

            /** 脚注列表 */
            footnotes: row?.footnotes ?? "",

            /** 所属类 */
            class_id: row?.class_id ?? "",

            /** 所属版本 */
            version_id: row?.version_id ?? "",
            versionList: versionList.value ?? [],
            classList: classList.value ?? []
          },
          ref: formRef
        }),
      beforeSure: async (done, {}) => {
        const FormData = formRef.value.newFormInline;
        if (title === "Add") {
          let addForm = {
            /** 父编码 */
            parent_id: FormData.parent_id ?? "",

            /** 编码 */
            htsno: FormData.htsno.replace(/\D/g, "") ?? "",

            /** 层级 */
            indent: FormData.indent ?? "",

            /** 描述 */
            description: FormData.description ?? "",

            /** 单位列表 */
            units: FormData.units ?? "",

            /** 通用税率 */
            general: FormData.general ?? "",

            /** 特殊税率，适用于特定国家或地区 */
            special: FormData.special ?? "",

            /** 其他税率 */
            other: FormData.other ?? "",

            /** 配额数量 */
            quota_quantity: FormData.quota_quantity ?? "",

            /** 附加税 */
            additional_duties: FormData.additional_duties ?? "",

            /** 脚注列表 */
            footnotes: FormData.footnotes ?? "",

            /** 所属类 */
            class_id: FormData.class_id ?? "",

            /** 所属版本 */
            version_id: FormData.version_id ?? ""
          };
          const res = await postAddHtsItemAPI(addForm);
          if (res.success) {
            done();
            await onSearch();
          }
          message(res.msg, { type: res.success ? "success" : "error" });
        } else {
          let updateForm = {
            /** 父编码 */
            parent_id: FormData.parent_id ?? "",

            /** 编码 */
            htsno: FormData.htsno.replace(/\D/g, "") ?? "",

            /** 层级 */
            indent: FormData.indent ?? "",

            /** 描述 */
            description: FormData.description ?? "",

            /** 单位列表 */
            units: FormData.units ?? "",

            /** 通用税率 */
            general: FormData.general ?? "",

            /** 特殊税率，适用于特定国家或地区 */
            special: FormData.special ?? "",

            /** 其他税率 */
            other: FormData.other ?? "",

            /** 配额数量 */
            quota_quantity: FormData.quota_quantity ?? "",

            /** 附加税 */
            additional_duties: FormData.additional_duties ?? "",

            /** 脚注列表 */
            footnotes: FormData.footnotes ?? "",

            /** 所属类 */
            class_id: FormData.class_id ?? "",

            /** 所属版本 */
            version_id: FormData.version_id ?? ""
          };
          const res = await putUpdateHtsItemAPI(updateForm, row.id);
          if (res.success) {
            done();
            await onSearch();
          }
          message(res.msg, { type: res.success ? "success" : "error" });
        }
      }
    });
  };

  /**
   * 获取版本列表
   */
  const getVersions = async () => {
    const res = await getVersionListAPI({ page: 1, pageSize: 99999 });
    if (res.success) {
      versionList.value = res.data.result;
    }
  };
  /**获取编码类别 */
  const getClassList = async () => {
    const res = await getHtsClassListAPI({ page: 1, pageSize: 9999 });
    if (res.success) {
      classList.value = res.data.result;
    }
  };

  /**
   * 页面加载执行
   */
  onMounted(async () => {
    await onSearch();
    await getClassList();
    await getVersions();
  });
  return {
    form,
    dataList,
    fileList,
    loading,
    pagination,
    uploadStatus,
    columns,
    versionList,
    classList,
    showUploadArea,
    version_id,
    class_id,
    beforeRemove,
    beforeUpload,
    handleUpload,
    selectedNum,
    openDialog,
    onSearch,
    resetForm,
    handleDelete,
    handleSizeChange,
    handleCurrentChange,
    handleSelectionChange,
    onSelectionCancel,
    onbatchDel
  };
};
