import dayjs from "dayjs";
import editForm from "../components/form.vue";
import { message } from "@/utils/message";
import { type Ref, ref, reactive, onMounted, h, toRaw } from "vue";
import type { VersionInfo } from "types/hts";
import type { PaginationProps } from "@pureadmin/table";
import { addDialog } from "@/components/ReDialog";
import {
  deleteVersionAPI,
  deleteVersionListAPI,
  getVersionListAPI,
  postAddVersionAPI,
  putUpdateVersionAPI
} from "@/api/hts";
import { getKeyList } from "@pureadmin/utils";
import { $t, transformI18n } from "@/plugins/i18n";

export const useVersion = (tableRef: Ref) => {
  /**
   * 查询表单
   */
  const form = reactive({
    /** 版本号 */
    version: "",

    /** 版本日期 */
    date: "",

    /** 下载地址 */
    url: ""
  });
  /**
   * 表单Ref
   */
  const formRef = ref(null);
  /**
   * 数据列表
   */
  const dataList = ref<VersionInfo[]>([]);
  /**
   * 加载状态
   */
  const loading = ref(true);
  /**
   * 已选数量
   */
  const selectedNum = ref<number>(0);
  /**
   * 分页参数
   */
  const pagination = reactive<PaginationProps>({
    total: 0,
    pageSize: 10,
    currentPage: 1,
    background: true,
    pageSizes: [10, 20, 30, 40, 50]
  });

  /**
   * 表格列设置
   */
  const columns: TableColumnList = [
    {
      label: transformI18n($t("tag.checkColumn")), // 如果需要表格多选，此处label必须设置
      type: "selection",
      fixed: "left",
      reserveSelection: true // 数据刷新后保留选项
    },
    {
      label: transformI18n($t("HTSversion.tag.version")),
      prop: "version",
      hide: false
    },

    {
      label: transformI18n($t("HTSversion.tag.date")),
      prop: "date",
      hide: false
    },

    {
      label: transformI18n($t("HTSversion.tag.url")),
      prop: "url",
      hide: false
    },
    {
      label: transformI18n($t("tag.createTime")),
      prop: "create_time",
      hide: false,
      formatter: ({ create_time }) =>
        dayjs(create_time).format("YYYY-MM-DD HH:mm:ss")
    },

    {
      label: transformI18n($t("tag.updateTime")),
      prop: "update_time",
      hide: false,
      formatter: ({ update_time }) =>
        dayjs(update_time).format("YYYY-MM-DD HH:mm:ss")
    },

    {
      label: transformI18n($t("buttons.Operation")),
      fixed: "right",
      width: 220,
      slot: "operation"
    }
  ];
  /**
   * 初次查询
   */
  const onSearch = async () => {
    loading.value = true;
    const res = await getVersionListAPI({
      page: pagination.currentPage,
      pageSize: pagination.pageSize,
      ...toRaw(form)
    });
    if (res.success) {
      dataList.value = res.data.result;
      pagination.total = res.data.total;
      pagination.currentPage = res.data.page;
      pagination.pageSize = res.data.pageSize;
    }
    message(res.msg, {
      type: res.success ? "success" : "error"
    });
    loading.value = false;
  };
  /**
   * 重置表单
   * @param formEl 表单ref
   * @returns
   */
  const resetForm = async (formEl: any) => {
    if (!formEl) return;
    formEl.resetFields();
    await onSearch();
  };
  /**
   * 处理删除
   * @param row
   */
  const handleDelete = async (row: VersionInfo) => {
    const res = await deleteVersionAPI(row.id);
    if (res.success) {
      onSearch();
    }
    message(res.msg, {
      type: res.success ? "success" : "error"
    });
  };
  /**
   * 处理每页数量变化
   */
  const handleSizeChange = async (val: number) => {
    loading.value = true;
    const res = await getVersionListAPI({
      page: pagination.currentPage,
      pageSize: val,
      ...toRaw(form)
    });
    if (res.success) {
      dataList.value = res.data.result;
      pagination.total = res.data.total;
      pagination.currentPage = res.data.page;
      pagination.pageSize = res.data.pageSize;
    }
    message(res.msg, {
      type: res.success ? "success" : "error"
    });
    loading.value = false;
  };

  /**
   * 处理页码变化
   * @param val
   */
  const handleCurrentChange = async (val: number) => {
    loading.value = true;
    const res = await getVersionListAPI({
      page: val,
      pageSize: pagination.pageSize,
      ...toRaw(form)
    });
    if (res.success) {
      dataList.value = res.data.result;
      pagination.total = res.data.total;
      pagination.currentPage = res.data.page;
      pagination.pageSize = res.data.pageSize;
    }
    message(res.msg, {
      type: res.success ? "success" : "error"
    });
    loading.value = false;
  };
  /** 当CheckBox选择项发生变化时会触发该事件 */
  const handleSelectionChange = async (val: any) => {
    selectedNum.value = val.length;
    // 重置表格高度
    tableRef.value.setAdaptive();
  };

  /** 取消选择 */
  const onSelectionCancel = async () => {
    selectedNum.value = 0;
    // 用于多选表格，清空用户的选择
    tableRef.value.getTableRef().clearSelection();
  };
  /**
   * 批量删除
   */
  const onbatchDel = async () => {
    // 返回当前选中的行
    const curSelected = tableRef.value.getTableRef().getSelectionRows();
    const res = await deleteVersionListAPI(getKeyList(curSelected, "id"));
    if (res.success) {
      message(res.msg, {
        type: "success"
      });
      tableRef.value.getTableRef().clearSelection();
      onSearch();
    } else {
      message(res.msg, { type: "error", duration: 5000 });
    }
  };
  const openDialog = async (title = "Add", row?: VersionInfo) => {
    addDialog({
      title: transformI18n($t(`buttons.${title}`)),
      props: {
        formInline: {
          /** 方式 */
          title: title,

          /** 主键 */
          id: row?.id ?? "",

          /** 创建时间 */
          create_time: row?.create_time ?? "",

          /** 更新时间 */
          update_time: row?.update_time ?? "",

          /** 版本号 */
          version: row?.version ?? "",

          /** 版本日期 */
          date: row?.date ?? "",

          /** 下载地址 */
          url: row?.url ?? ""
        }
      },
      width: "45%",
      draggable: true,
      fullscreenIcon: true,
      closeOnClickModal: false,
      contentRenderer: () =>
        h(editForm, {
          formInline: {
            // /** 方式 */
            // title: title,

            /** 主键 */
            id: row?.id ?? "",

            /** 创建时间 */
            create_time: row?.create_time ?? "",

            /** 更新时间 */
            update_time: row?.update_time ?? "",

            /** 版本号 */
            version: row?.version ?? "",

            /** 版本日期 */
            date: row?.date ?? "",

            /** 下载地址 */
            url: row?.url ?? ""
          },
          ref: formRef
        }),
      beforeSure: async (done, {}) => {
        const FormData = formRef.value.newFormInline;
        if (title === "Add") {
          let addForm = {
            /** 版本号 */
            version: FormData.version ?? "",

            /** 版本日期 */
            date: FormData.date ?? "",

            /** 下载地址 */
            url: FormData.url ?? ""
          };
          const res = await postAddVersionAPI(addForm);
          if (res.success) {
            done();
            await onSearch();
          }
          message(res.msg, { type: res.success ? "success" : "error" });
        } else {
          let updateForm = {
            /** 版本号 */
            version: FormData.version ?? "",

            /** 版本日期 */
            date: FormData.date ?? "",

            /** 下载地址 */
            url: FormData.url ?? ""
          };
          console.log(updateForm);
          const res = await putUpdateVersionAPI(updateForm, row.id);
          if (res.success) {
            done();
            await onSearch();
          }
          message(res.msg, { type: res.success ? "success" : "error" });
        }
      }
    });
  };
  /**
   * 页面加载执行
   */
  onMounted(async () => {
    await onSearch();
  });
  return {
    form,
    dataList,
    loading,
    pagination,
    columns,
    selectedNum,
    openDialog,
    onSearch,
    resetForm,
    handleDelete,
    handleSizeChange,
    handleCurrentChange,
    handleSelectionChange,
    onSelectionCancel,
    onbatchDel
  };
};
